'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _fs = require('fs');

var _fs2 = _interopRequireDefault(_fs);

var _npm = require('npm');

var _npm2 = _interopRequireDefault(_npm);

var _path = require('path');

var _path2 = _interopRequireDefault(_path);

var _yargsParser = require('yargs-parser');

var _yargsParser2 = _interopRequireDefault(_yargsParser);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function BabelInstaller() {
  var _this = this;

  var options = arguments.length <= 0 || arguments[0] === undefined ? {
    checkBabelRC: true,
    config: {
      saveDev: true,
      saveExact: true
    }
  } : arguments[0];

  if (!(this instanceof BabelInstaller)) return new BabelInstaller(options);

  ['checkBabelRC', 'config'].forEach(function (k) {
    return _this[k] = options[k];
  });
  this.pkgNamesFromCLI = (0, _yargsParser2.default)(process.argv.slice(2))._;
}

BabelInstaller.prototype.load = function (next) {
  var _this2 = this;

  if (this.loaded) return next();
  _npm2.default.load(function (error) {
    if (!error) {
      Object.keys(_this2.config).forEach(function (k) {
        return _npm2.default.config.set(k, _this2.config[k]);
      });
      _this2.loaded = true;
    }
    return next(error);
  });
  return this;
};

BabelInstaller.prototype.saveToBabel = function (names) {
  var fileName = '.babelrc';
  return writeJSON(fileName, names.reduce(function (babelrc, name) {
    var prefix = undefined;
    Object.keys(babelPrefixes).forEach(function (prfx) {
      name.indexOf(babelPrefixes[prfx]) === 0 && (prefix = prfx);
    });
    babelrc[prefix] || (babelrc[prefix] = []);
    var savableName = name.replace(babelPrefixes[prefix], '');
    if (babelrc[prefix].indexOf(savableName) !== -1) {
      console.error(savableName + ' (' + name + ') was already saved to ' + fileName);
    } else {
      babelrc[prefix].push(savableName);
    }
    return babelrc;
  }, readJSON(fileName)));
};

BabelInstaller.prototype.installAndDeclare = function (names, next) {
  var _this3 = this;

  typeof names === 'function' && (next = names) && (names = this.pkgNamesFromCLI);

  if (!names.length) {
    throw new Error('We need some package names in order to install anything dudette (or dude)!');
  }

  var nameMapper = function nameMapper(name) {
    return name.indexOf('@') !== -1 ? name : name + '@latest';
  };
  return this.load(function (loadError) {
    if (loadError) return console.error(loadError);
    return _npm2.default.commands.install(names.map(nameMapper), function (installError) {
      if (!installError) {
        _this3.saveToBabel(names);
        Object.keys(_this3.config).forEach(function (k) {
          return _npm2.default.config.set(k, undefined);
        });
      }
      return next ? next(installError) : console.error(installError);
    });
  });
};

// Declared but uninstalled plugins & transforms
BabelInstaller.prototype.getUninstalled = function () {
  var babelrc = readJSON('.babelrc');
  var pkg = readJSON('package.json');
  var devPkgNames = Object.keys(pkg.devDependencies);
  var missingPackages = [];
  Object.keys(babelPrefixes).forEach(function (prfx) {
    if (babelrc[prfx]) {
      babelrc[prfx].forEach(function (shortName) {
        var fullName = '' + prfx + shortName;
        if (devPkgNames.indexOf(fullName) === -1) {
          missingPackages.push(fullName);
        }
      });
    }
  });
  return missingPackages;
};

BabelInstaller.prototype.smartNames = function () {
  return this.pkgNamesFromCLI.concat(this.checkBabelRC ? this.getUninstalled() : []);
};

function prependCwd(fileName) {
  return _path2.default.join(process.cwd(), fileName);
}

function readJSON(fileName) {
  return JSON.parse(_fs2.default.readFileSync(prependCwd(fileName), 'utf8'));
}

function writeJSON(fileName, json) {
  var string = JSON.stringify(json, null, 2) + '\n';
  return _fs2.default.writeFileSync(prependCwd(fileName), string, 'ascii');
}

var babelPrefixes = {
  plugins: 'babel-plugin-',
  preset: 'babel-preset-'
};

// CLI up
if (!module.parent) {
  BabelInstaller().installAndDeclare(function (installError) {
    return installError ? console.error(installError) : console.info('Install succeeded');
  });
}

exports.default = BabelInstaller;
module.exports = exports['default'];
